/**********************************************************************************************
       Module:  csbmva_build_pool_scr
     Function:  This macro build the pool for SCORE model

      Authors:  BIS Team
         Date:  23 Nov 2010
          SAS:  9.2
    Called-by:	Build Screen. 

        Calls:  NA

Datasets used:  _<pool_scheme_no>_build_pool_in_ds
				Range table of score
 
        Logic:  Assign the records from the input dataset in their respective pools

   Parameters: 
		INPUT: 
				range_table_lib = Libname of input dataset,
				range_table_nm  = Range table name,
                pool_scheme_no = pool_scheme_no,
				source_ds = _<pool_scheme_no>_build_pool_in_ds     ,
                output_ds_lib = libname od output dataset,
				output_ds = Name of output dataset
**********************************************************************************************/

%macro csbmva_build_pool_scr(range_table_lib = ,range_table_nm = ,
                             pool_scheme_no = ,source_ds =      ,
                             output_ds_lib = ,output_ds = );


%let out_lib = work;									
*==================================================;
* creating temporary range table ;
*==================================================;

proc sql noprint;
  create table work.build_range as 
	select monotonic() as pool_seq_no, from_score, to_score,
	put(from_score, &CS_DEC_FMT.)||"%sysfunc(sasmsg(smd_ds.bismsg,CRS.BUILD_SCHEME_SCORE_POOL.COLUMN_RANGE_SCORE.1, noquote))"||
	put(to_score, &CS_DEC_FMT.) 
	as pool_name
	from &range_table_lib..&range_table_nm.;
quit;

*==================================================;
* Getting details of pool_scheme ;
*==================================================;

proc sql noprint;
  select pd_method_type_cd, pd_model_rk
	into
	:pd_method, :pd_model_key
  from  &csb_apdm_libref..pool_scheme_defn
  where pool_scheme_no=&pool_scheme_no.;
quit;


*==================================================;
* Getting pools & Assigned scr ;
*==================================================;

%let pd_mtd=%quote(%'&pd_method%');
%if (%kupcase(&pd_mtd) eq %kupcase(&LONG_RUN_MAX_TYPE.)) or
	(%kupcase(&pd_mtd) eq %kupcase(&LONG_RUN_AVG_TYPE.)) 

%then %do; /*START - LONGRUN METHOD */

  *--------------------------------------------;
  * Subsetting source_ds ;
  *--------------------------------------------;

  proc sql noprint;
  	select scan(abt_dts,1,',') into :build_dt
	from &csb_apdm_libref..build_pool_abt_status
	where pool_scheme_no = &pool_scheme_no.;
  quit;
  %let build_dt = &build_dt;

  proc sql;
  	create table temp_source as
		select * from pool.&source_ds
		where year_dt ne "&build_dt"d;
 quit;

/*  proc sort data = pool.&source_ds out = temp_source;*/
/*    by year_dt;*/
/*  run;*/
/**/
/*  data temp_source;*/
/*  	set temp_source;*/
/*	by year_dt;*/
/*	if last.year_dt then delete;*/
/*  run;*/

  *--------------------------------------------;
  * calculating assigned SCR ;
  *--------------------------------------------;

  proc sql;
	create table &out_lib..temp_lr_scr AS
	  select year_dt, pool_seq_no,
	  		 max(score_points) as score_points_yr,
		     sum(actual_good_bad) as NO_OF_BADS,
			 count (*) as NO_OF_records,
			 calculated NO_OF_BADS / calculated NO_OF_records as pd_for_pool
	  from temp_source, work.build_range
	  where score_points > from_score and score_points <= to_score 
	  group by year_dt, pool_seq_no;
  quit;

  %if (%kupcase(&pd_mtd) eq %kupcase(&LONG_RUN_MAX_TYPE.)) or 
      (%kupcase(&pd_mtd) eq %kupcase(&MAX_EST_TYPE.)) 
  %then %do;

  	proc sql;
	  create table &out_lib..max_lr_scr AS
	  	select a.pool_seq_no,
		  a.score_points_yr as score_points,
	      a.pd_for_pool as assigned_pd,
		  a.no_of_records as no_of_records
		from &out_lib..temp_lr_scr as a
		where pd_for_pool = (select max(pd_for_pool)
							 from &out_lib..temp_lr_scr as b
							 where a.pool_seq_no = b.pool_seq_no);

	quit;

	proc sort data = &out_lib..max_lr_scr;
	  by pool_seq_no no_of_records;
	run;
	proc sort data = &out_lib..max_lr_scr out = &out_lib..lr_scr nodupkey;
	  by pool_seq_no;
	run;

  %end;
  %else %do;
    proc sql;
      create table &out_lib..lr_scr AS
	    select a.pool_seq_no,
		  avg(a.pd_for_pool) as assigned_pd,
		  max(a.score_points_yr) as score_points,
	      avg(a.no_of_records) as no_of_records
	    from &out_lib..temp_lr_scr as a
  	    group by a.pool_seq_no;
	quit;

  %end;

 proc sql;
  	create table &output_ds_lib..&output_ds as
    select &POOL_sCHEME_NO. AS POOL_SCHEME_NO, a.pool_seq_no, a.pool_name,
    a.from_score, a.to_score,. as from_pd,. as to_pd,
	coalesce(b.no_of_records,0) as no_of_records, b.score_points,b.ASSIGNED_PD,
	0 AS Exposure_flag 
	from work.build_range as a left join &out_lib..lr_scr as b
	on (a.pool_seq_no = b.pool_seq_no);
  quit;

%end;/*END - LONGRUN METHOD */
%else %do;/*START - ESTIMATED METHOD */

  /* Mapping pools */
  proc sql;
	create table &out_lib..temp_estimated_ds AS
	  select a.pool_seq_no, score_points,
               estimated_pd
	  from pool.&source_ds , work.build_range a
	  where score_points > from_score and score_points <= to_score ;
  quit;

  /* Grouping data on pools */
  proc sql;
    create table &out_lib..temp_estimated_ds2 AS
	  select pool_seq_no,count(*) as no_of_records,
		  %if %kupcase(&pd_mtd) eq %kupcase(&MAX_EST_TYPE.) %then %do;
            max(estimated_pd) as assigned_pd,
		  %end;
          %else %do;
		    avg(estimated_pd) as assigned_pd,
		  %end;
		  max(score_points) as score_points
	  from &out_lib..temp_estimated_ds
	  group by pool_seq_no;
  quit;

  /* Creating output table */
  proc sql;
    create table &output_ds_lib..&output_ds as
      select &pool_scheme_no. as pool_scheme_no, a.pool_seq_no, a.pool_name,
        a.from_score, a.to_score,. as from_pd,. as to_pd,
	    coalesce(b.no_of_records,0) as no_of_records, b.score_points,
        b.assigned_pd,
		0 as exposure_flag
	  from work.build_range as a left join &out_lib..temp_estimated_ds2 as b
	  on (a.pool_seq_no = b.pool_seq_no);
  quit;
  
%end;/*END - ESTIMATED METHOD */

proc sql;
select count(*) into :cnt
from  &csb_apdm_libref..pool_defn
where pool_scheme_no = &pool_scheme_no.;
quit;

%let cnt = &cnt;

/*%if &cnt. eq 0 %then*/
/*%do;*/
	
	proc sql noprint;
		select count(actual_good_bad) into :cnt_dlq
		from pool.&source_ds.
		where actual_good_bad = 1;

		select min(score_points), max(score_points) into :from_dlq, :to_dlq
		from pool.&source_ds.
		where actual_good_bad = 1;

		select max(pool_seq_no) into :pool_seq
		from &output_ds_lib..&output_ds
		;
	quit;



	data delinquent;
	POOL_SCHEME_NO= &POOL_SCHEME_NO. ;
	POOL_SEQ_NO= sum(&pool_seq. ,1) ;
	POOL_NAME= put(&from_dlq., &CS_DEC_FMT.)||"%sysfunc(sasmsg(smd_ds.bismsg,CRS.BUILD_SCHEME_SCORE_POOL.COLUMN_RANGE_DELINQUENT.1, noquote))"||put(&to_dlq., &CS_DEC_FMT.)	;
	FROM_SCORE_NO= &from_dlq.;
	TO_SCORE_NO= &to_dlq. ;
	ASSIGNED_PD= 1;
	no_of_records= &cnt_dlq.;
	from_pd=1;
	to_pd=1;
	run;

	
	proc sql;
	alter table &output_ds_lib..&output_ds
	modify pool_name char(150);
	quit;

	proc sql;
	insert into &output_ds_lib..&output_ds 
	(POOL_SCHEME_NO, POOL_SEQ_NO, POOL_NAME, FROM_SCORE, TO_SCORE, ASSIGNED_PD, NO_OF_RECORDS,
     from_pd,to_pd,EXPOSURE_FLAG)
	select
	POOL_SCHEME_NO, POOL_SEQ_NO, POOL_NAME, FROM_SCORE_NO, TO_SCORE_NO, ASSIGNED_PD, no_of_records,
	from_pd,to_pd,1 as Exposure_Flag
	from delinquent;
	quit;


/*%END; */
/* END FOR DELINQUENT */
  
 
%mend csbmva_build_pool_scr;
