/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_feed_pd.sas
Function       : This macro creates the feed table for PD Model on pool level.

Authors        :  CSB Team
Date           :  17AUG2010
SAS            :  9.2

Called-by      :  %Csbmva_generate_feed Macro. 
Calls          :  %RCSET macro to capture the return code for error handling.

Input Datasets :  _<MODEL_RK>_<SCORE_TIME_SK>_PD_FACT (CSB FACT library)
                  _<MODEL_RK>_DEV_FACT (CSB FACT library)
                  MODEL_X_POOL (CSB BRIDGE library)
                  APPLICATION_DIM (DIM library)
                  POOL_DIM (CSB DIM library)

Output Datasets:  _<MODEL_RK>_<SCORE_TIME_SK>_PD_POOL_FD (CSB FACT library)

Logic          :  1. Get the RATE from MODEL_X_POOL table and join with PD fact.
                  2. Group the above table based on pools and derive the variable related to
                     actual sample.
                  3. Get the dev count from MODEL_DEV_FACT.
                  4. Join with the POOL_DIM to get the POOL_SEQ_NO and POOL_NAME.

Parameters     :  MODEL_TIME_KEY     -> Model_RK & Score_Time_SK concatenated by underscore(_).
                  MODEL_TYPE         -> Type code of model e.g. PDA, PDO, LGD, CCF.
                  INPUT_DS_LIB       -> Library where PD fact is present.
                  INPUT_DS_NM        -> Name of PD Fact.
                  OUT_FEED_LIB       -> Library where feed table will be created.
                  FEED_DS_NM_POOL    -> Name of the feed table.
		  ISDEVELOPMENTCYCLE ->	Whether model is in development phase or scoring
						(possible values : true, false)

**********************************************************************************************/

%macro csbmva_feed_pd(model_key    =  ,time_key        =  ,
                      input_ds_lib =  ,input_ds_nm     =  ,
                      out_feed_lib =  ,feed_ds_nm_pool =  ,
                      model_type   =  ,isDevelopmentCycle = );

%let model_key = &model_key;
%let model_type = &model_type;

*==================================================================;
* Creating local macro variables ;
*==================================================================;

%local  period_end_dttm  m_level_sk
		scheme_type		 pool_scheme_sk
		mm_ext_key		factor
		offset;

*==================================================================;
* Mapping time_sk to datetime value ;
*==================================================================;

data _null_;
  a = put(&time_key,cs_period_last_dttm_fmt.);
  call symput('period_end_dttm',a);
run;

*==================================================================;
* Finding the model_key from model_master_extension ;
*==================================================================;

%if &isDevelopmentCycle = true %then %do;
	%let mm_ext_key = &model_key;
%end;
%else %do;
	proc sql noprint;
		select source_model_master_key into :mm_ext_key
		from dim.analytical_model_dim
		where model_rk = &model_key;
	quit;
	%let mm_ext_key = &mm_ext_key;
%end;

*==================================================================;
* finding the FACTOR and OFFSET of the model ;
*==================================================================;

%let factor = .;
%let offset = .;
proc sql noprint;
  select factor, offset_rt into :factor, :offset
  from &dabt_lib..model_master_extension
  where model_key = &mm_ext_key;
quit;

%if %sysevalf(&factor) = . or %sysevalf(&offset) = . %then %do;
	%put ERROR: Invalid data value for FACTOR and OFFSET;
	%bankfdn_rcset(9999);
	%return;
%end;

*==================================================================;
* finding the pool_schem_type of the model ;
*==================================================================;

proc sql noprint;
  select a.level_sk, a.pool_scheme_sk,
		 kupcase(b.pool_scheme_type_cd)
    into  :m_level_sk, :pool_scheme_sk, :scheme_type
  from &dabt_lib..model_master_extension as a,&csb_apdm_libref..pool_scheme as b
  where a.pool_scheme_sk = b.pool_scheme_sk
  and a.model_key = &mm_ext_key;
quit;

%bankfdn_rcset(&sqlrc);
%let scheme_type = &scheme_type;
%let scheme_type = %str(%')&scheme_type%str(%');

*==================================================================;
* Processing POOL_DIM ;
*==================================================================;

proc sql;
  create table pool_dim_ext_tmp1 as
    select * from &csb_apdm_libref..pool_dim
	where %if &isDevelopmentCycle = true %then %do;
			valid_end_dttm = &valid_high_dttm;
		  %end;
		  %else %do;
			valid_start_dttm <= &period_end_dttm <= valid_end_dttm;
		  %end;
quit;

%if &model_type=&PD_ONG_MODEL %then %do;

  proc sort data = pool_dim_ext_tmp1 out = pool_dim_ext_tmp2;
    by pd_from_no pool_sk;
	where pool_scheme_sk = &pool_scheme_sk;
  run;
  data pool_dim_ext;
  	length pool_name $300;
    set pool_dim_ext_tmp2(drop = pool_name);
	by pd_from_no;
	if first.pd_from_no;

	if pd_to_no = 1 then 
		score_from_no = 0;
	else 
		score_from_no = log((1-pd_to_no)/pd_to_no) * &factor + &offset;

	if pd_from_no = 0 then 
		score_to_no = &SCORE_POINTS_UPPER_BOUND;
	else 
		score_to_no = log((1 - pd_from_no)/pd_from_no) * &factor + &offset;

	score_from_no = int(score_from_no);
	score_to_no = int(score_to_no);

	pool_name = put(score_from_no, &CS_DEC_FMT.)||
	  	  "%sysfunc(sasmsg(smd_ds.bismsg,CRS.BUILD_SCHEME_SCORE_POOL.COLUMN_RANGE_SCORE.1, noquote))"||
		  put(score_to_no, &CS_DEC_FMT.);

  run;

%end;
%else %do;
  data pool_dim_ext;
    set pool_dim_ext_tmp1;
  run;
%end;

*==================================================================;
* Assigning pool_sk to fact table ;
*==================================================================;
						
proc sql;
  create table &debug_lib..feed_table_actual as
    select  record_rk, score_points, 
      		actual_good_bad,  b.pool_sk
    from &input_ds_lib..&input_ds_nm as a
           right outer join
         pool_dim_ext as b
	on (  b.score_from_no < a.score_points <= b.score_to_no )
    where b.pool_scheme_sk = &pool_scheme_sk;
quit;
%bankfdn_rcset(&sqlrc);

/* Grouping the data at POOL */
proc sql;
  create table &debug_lib..feed_table_pool_actual as
    select a.pool_sk,
      count(a.record_rk) as no_of_records_actual,
	  coalesce(sum(a.actual_good_bad),0) as no_of_actual_bads
	from &debug_lib..feed_table_actual as a
  	     %if &model_type=%str(&PD_APP_MODEL) %then %do;
           inner join dim.application_dim(where = ( kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT and valid_end_dttm = &valid_high_dttm. )) as b 
		   on (a.record_rk = b.application_rk)
		 %end;
	group by a.pool_sk;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================================;
* Assigning Pools to Dev sample based on current pools ;
*==================================================================;

proc sql;
  create table &debug_lib..feed_table_dev as 
    select b.pool_sk, record_rk, score_points, estimated_pd
	from &cs_fact_lib.._&mm_ext_key._dev_fact as a
           right outer join
         pool_dim_ext as b
	on ( b.score_from_no < a.score_points <= b.score_to_no )
    where b.pool_scheme_sk = &pool_scheme_sk;
quit;
%bankfdn_rcset(&sqlrc);

/* Getting no. of records in DEV */
proc sql;
  create table &debug_lib..feed_table_pool_dev as
    select pool_sk, count(record_rk) as no_of_records_dev,
		   max(score_points) as max_score_points,
		   avg(estimated_pd) as estimated_pd
    from &debug_lib..feed_table_dev
	group by pool_sk
	order by max_score_points;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================================;
* Derive POOL_NAME ;
*==================================================================;

data pool_details(keep = pool_sk pool_name);
  set pool_dim_ext(where = (pool_scheme_sk = &pool_scheme_sk));
run;

*==================================================================;
* Creating pool level feed ;
*==================================================================;

proc sql;

	CREATE TABLE &out_feed_lib..&feed_ds_nm_pool
	(
	POOL_SK NUMERIC(8) FORMAT = 12. INFORMAT = 12. NOT NULL ,
	POOL_SEQ_NO NUMERIC(8) ,
	POOL_NAME CHARACTER(300) FORMAT = $300. INFORMAT = $300.  ,
	NO_OF_RECORDS_DEV NUMERIC(8)  ,
	%if &model_type=%str(&PD_APP_MODEL) %then %do;
		NO_OF_RECORDS_ACTUAL_ALL NUMERIC(8)  ,
	%end;
	NO_OF_RECORDS_ACTUAL NUMERIC(8)  ,
	NO_OF_ACTUAL_BADS NUMERIC(8)  ,
	MAX_SCORE_POINTS NUMERIC(8) ,
	ESTIMATED_PD  NUMERIC(8) ,
	MODEL_RK NUMERIC(8) FORMAT = 12. INFORMAT = 12. NOT NULL ,
	LEVEL_SK NUMERIC(8) NOT NULL ,
    SCORE_TIME_SK NUMERIC(8) FORMAT = 12. INFORMAT = 12. NOT NULL 
	);

    create unique index xpk_feed_pd on &out_feed_lib..&feed_ds_nm_pool(model_rk,score_time_sk,pool_sk);

quit;
%bankfdn_rcset(&sqlrc);

%if &model_type=%str(&PD_APP_MODEL) %then %do;

	proc sql;
	  create table &debug_lib.._&model_key._&time_key._feed_table_act_app as
	    select
			a.pool_sk,
	      	count(a.record_rk) as no_of_records_actual_all
		  	from &debug_lib..feed_table_actual as a
			group by a.pool_sk;
	quit;
%end;
proc sql;
  insert into &out_feed_lib..&feed_ds_nm_pool 
    select a.POOL_SK, monotonic(), a.POOL_NAME,
      coalesce(c.no_of_records_dev,0) as NO_OF_RECORDS_DEV,
      	%if &model_type=%str(&PD_APP_MODEL) %then %do;
		coalesce(d.no_of_records_actual_all,0) as no_of_records_actual_all,
	%end;
	  coalesce(b.no_of_records_actual,0) as NO_OF_RECORDS_ACTUAL,
	  coalesce(b.no_of_actual_bads,0) as NO_OF_ACTUAL_BADS,
	  MAX_SCORE_POINTS,
	  ESTIMATED_PD,
	  &model_key as MODEL_RK,
	  &m_level_sk as LEVEL_SK,
	  &time_key as SCORE_TIME_SK
	from work.pool_details a
		left join &debug_lib..feed_table_pool_actual as b
		on (a.pool_sk=b.pool_sk)
		left join &debug_lib..feed_table_pool_dev as c
		on (a.pool_sk = c.pool_sk)
		%if &model_type=%str(&PD_APP_MODEL) %then %do;
			left join &debug_lib.._&model_key._&time_key._feed_table_act_app d
			on ( a.pool_sk = d.pool_sk )
		%end;
	;
quit;
%bankfdn_rcset(&sqlrc);

%mend csbmva_feed_pd;
