/*
Macro Name	: csbmva_ks
Function	: Generates Confusion Matrix for every cut-off for the given input.
			  The data-set generated will be input to calculation various statistics 
Created by	: BIS Team
Date		: aug , 2010
SAS Version	: 9.2

Called-by	:Data preparation stored proc. This macro is called before calculation of various statistic
			 like ROC, AUC, Pietra, BER etc. Applicable for ESTIMATED_PD based pooling scheme.
Calls		:None

Parameters	: 

input_ds		: Name of the input data set
output_ds		: Name of the data set which stores confusion matrix.
dist_type		: Type of Variable (Actual /Estimated)
				  Possible Values ACT/EST	
Input Dataset(s) Format:

--------------------------------------------------------------------------
Name				Type		Format
--------------------------------------------------------------------------
Pool_Name			Character 	Default Formatting	
Pool_Seq_no	 		Number		Default Formatting
Pool_Sk	 		Number		Default Formatting
score_time_sk           Number		Default Formatting
level_sk                Number		Default Formatting
Model_sk                 Number		Default Formatting
Model_rk                 Number		Default Formatting
No_Of_records_Actual	Number		Default Formatting
No_of_records_Dev	 	Number 		Default Formatting
estimated_PD	 				Number		Default Formatting
Max_Score_Points	 	Number		Default Formatting
No_Of_Actual_Bads	Number		Default Formatting	
--------------------------------------------------------------------------


Output Dataset for ks calculation and graph  
Format:
output_graph_ds
-------------------------------------------------------------------------
Name		Type		Format
------------------------------------------------------------------------
score		Numeric		Default Formatting			 
Dist_good	Numeric		Default Formatting
Dist_bad	Numeric		Default Formatting
ks			Numeric		Default Formatting
------------------------------------------------------------------------
Logic:
1. Compute propotion of goods which are treated as bads in Dist_goods .
2. Compute proporion of bads which are treated as bads with Dist_bads 
3. Difference between Dist_Bad and Dist good is Ks 
4. Generated data of dist_goods and dist_bads is used to calculate KS-Chart.

Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.

*/








%macro csbmva_ks(input_ds=,output_ds=,flag=);

/* Considers each pool_maximum as cutoff and computes the confusion matrix for each cut-off 
For all cut-offs in a table we will generate following table
		Actual 
	---------------
 	|Good	| Bad	|Total	|
--------------------|-------|
Bad	|BGood  |BBad	|Bads	|
-------------------	|-------|
Good|GGood	|GBad 	|Goods  |
-------------------	|-------|
*/
/*In few macro the exact location of change is not mentioned. Just file is known so 
  it is identified with the tag of hot fix as below*/
/*CS4.3 HF2 */

%if &flag = 1 %then %do;
data _NULL_;

name=put(time(),mmss8.1);

name1=kscan(name,1,":.");

name2=kscan(name,2,":.");

name3=kscan(name,3,":.");

name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));

call symput('log_name',"&LOG_PATH_CS"||"&sysmacroname"||ktrim(kleft(name4))||".log");

run;


%end;

data input;
set &input_ds;
No_of_bads_act = No_of_Actual_bads;
No_of_bads_est = ESTIMATED_PD* NO_OF_RECORDS_ACTUAL;
run;

data temp_score;
set input(keep=MAX_SCORE_POINTS);
rename MAX_SCORE_POINTS=score;
run;

proc sql;
	/*creating the temporary tables and  taking values above score points*/
	/*------- For creating First Row of the table for each cut-off 
	Bad	|BGood  |BBad	|Bads|
	-------------------	|-------|
	*/
	create table temp1 as
	select NO_OF_RECORDS_ACTUAL , No_Of_Bads_act, MAX_SCORE_POINTS,score
	from input,temp_score
	where MAX_SCORE_POINTS <= score;

	/*creating the tables for above score points required for joining*/
	create table Si_table as 
	select score, sum(NO_OF_RECORDS_ACTUAL )as Total_Accounts, sum(No_Of_Bads_act ) as TN_act,
		              sum(NO_OF_RECORDS_ACTUAL )- sum(No_Of_Bads_act ) as FP_act
	from temp1 group by score;


	/*creating the temporary tables and  taking values below score */
	/*------- For creating Second Row of the table for each cut-off ------------------------
	Good|GGood	|GBad 	|Goods  |
	-------------------	|-------|
	*/
	create table temp2 as
	select NO_OF_RECORDS_ACTUAL , No_Of_Bads_act,MAX_SCORE_POINTS,score
	from input,temp_score
	where MAX_SCORE_POINTS > score;

	/*creating the tables for below score points required for joining*/
	create table Si_table1 as
	select score, sum(NO_OF_RECORDS_ACTUAL )as Total_Accounts, sum(No_Of_Bads_act ) as FN_act,
	              sum(NO_OF_RECORDS_ACTUAL )- sum(No_Of_Bads_act ) as TP_act
	from temp2 group by score;


	/*For combining first and second row of confusion matrix use join*/
	create table conf_mat as 
	select si_table.score,Si_table1.FN_act,Si_table1.TP_act,Si_table.TN_act,Si_table.FP_act
	from Si_table left outer join Si_table1
	on si_table1.score= si_table.score;

	/*droping temporary tables which not used further*/	
	drop table temp1;
	drop table temp2;
	
	drop table si_table;
	drop table si_table1;

	/*uESTIMATED_PDating the missing values */
	update conf_mat SET FN_act=0 WHERE FN_act is missing; 
	update conf_mat SET TP_act=0 WHERE TP_act is missing;
	update conf_mat SET FP_act=0 WHERE FP_act is missing; 
	update conf_mat SET TN_act=0 WHERE TN_act is missing;
quit;


%let err_code=;

data output_ds;
	set conf_mat;
	var_tot_acc = TP_act+FP_act+TN_act+FN_act;
	var_no_bads = FN_act+TN_act;
run;
/***************************************************************/
%csbmva_alldefaults_nodefaults(output_ds,var_tot_acc,var_no_bads, err_code);
%if (&err_code ne 'NONE')%then 
%do;
/***************************************************************/

%csbmva_defdata(input_ds=output_ds,stat='KS_ACT',
		out_graph_ds =output_graph_ds_act,output_ds = ,out_stat_ds=);
data output_graph_ds_act;
set output_graph_ds_act;
diff_act =.;
maxval_act=.;
dgoods_act=.;
dbads_act=.;
run;

%end;
%else %do;

data output_graph_ds_act;
	set output_ds;
    Dist_good_act   = 1.0 -(tp_act/(tp_act + fp_act));
    Dist_bad_act 	=tn_act/(tn_act + fn_act);
    ks_act = abs(Dist_bad_act - Dist_good_act) ;/*Absolute function added as per the theory Hot Fix: Defect ID:S1142267*/
	keep score Dist_good_act Dist_bad_act  ks_act; 
run;


data output_graph_ds_act;
set output_graph_ds_act;
diff_act = Dist_Good_act - Dist_Bad_act; 
	maxval_act=.; dgoods_act=.; dbads_act=.;
run;

proc sql;
	create table maxtab_x
	as select diff_act, dist_good_act,  dist_bad_act, score 
	from output_graph_ds_act  
	where ks_act=(select max(ks_act) from output_graph_ds_act); /* To find the max diff and co-ords*/

	create table maxtab as
	  select * from maxtab_x
	  where score = (select min(score) from maxtab_x);

	update output_graph_ds_act 
        set maxval_act=(select diff_act from  maxtab),
	dgoods_act=(select dist_good_act from maxtab),
	dbads_act=(select dist_bad_act from  maxtab)

	where score=(select score from  maxtab);
		
	
quit;
%end;

/*estimate*/
	
proc sql;
	/*creating the temporary tables and  taking values above score points*/
	/*------- For creating First Row of the table for each cut-off 
	Bad	|BGood  |BBad	|Bads|
	-------------------	|-------|
	*/
	create table temp1 as
	select no_of_records_actual , No_Of_Bads_est, MAX_SCORE_POINTS,score
	from input,temp_score
	where MAX_SCORE_POINTS <= score;

	/*creating the tables for above score points required for joining*/
	create table Si_table as 
	select score, sum(no_of_records_actual )as Total_Accounts, sum(No_Of_Bads_est ) as TN_est,
		              sum(no_of_records_actual )- sum(No_Of_Bads_est ) as FP_est
	from temp1 group by score;


	/*creating the temporary tables and  taking values below score */
	/*------- For creating Second Row of the table for each cut-off ------------------------
	Good|GGood	|GBad 	|Goods  |
	-------------------	|-------|
	*/
	create table temp2 as
	select no_of_records_actual , No_Of_Bads_est,MAX_SCORE_POINTS,score
	from input,temp_score
	where MAX_SCORE_POINTS > score;

	/*creating the tables for below score points required for joining*/
	create table Si_table1 as
	select score, sum(no_of_records_actual )as Total_Accounts, sum(No_Of_Bads_est ) as FN_est,
	              sum(no_of_records_actual )- sum(No_Of_Bads_est ) as TP_est
	from temp2 group by score;


	/*For combining first and second row of confusion matrix use join*/
	create table conf_mat as 
	select si_table.score,Si_table1.FN_est,Si_table1.TP_est,Si_table.TN_est,Si_table.FP_est
	from Si_table left outer join Si_table1
	on si_table1.score= si_table.score;

	/*droping temporary tables which not used further*/	
	drop table temp1;
	drop table temp2;
	
	drop table si_table;
	drop table si_table1;

	/*uESTIMATED_PDating the missing values */
	update conf_mat SET FN_est=0 WHERE FN_est is missing; 
	update conf_mat SET TP_est=0 WHERE TP_est is missing;
	update conf_mat SET FP_est=0 WHERE FP_est is missing; 
	update conf_mat SET TN_est=0 WHERE TN_est is missing;
quit;



%let err_code=;

data output_ds;
	set conf_mat;
	var_tot_acc = TP_est+FP_est+TN_est+FN_est;
	var_no_bads = FN_est+TN_est;
run;
/***************************************************************/
%csbmva_alldefaults_nodefaults(output_ds,var_tot_acc,var_no_bads, err_code);
%if (&err_code ne 'NONE')%then 
%do;
/***************************************************************/

%csbmva_defdata(input_ds=output_ds,stat='KS_EST',
		out_graph_ds = output_graph_ds_est, output_ds = ,out_stat_ds=);

data output_graph_ds_est;
set output_graph_ds_est;
diff_est =.;
maxval_est=.;
dgoods_est=.;
dbads_est=.;
run;

%end;
%else %do;

data output_graph_ds_est;
	set output_ds;
    Dist_good_est   = 1.0 -(tp_est/(tp_est + fp_est));
    Dist_bad_est 	= tn_est/(tn_est + fn_est);
    ks_est = abs(Dist_bad_est - Dist_good_est) ;/*Absolute function added as per the theory Hot Fix: Defect ID:S1142267*/
	keep score Dist_good_est Dist_bad_est  ks_est; 
run;


data output_graph_ds_est;
set output_graph_ds_est;
diff_est = Dist_Good_est - Dist_Bad_est; 
	maxval_est=.; dgoods_est=.; dbads_est=.;
run;

proc sql;
	create table maxtab_x
	as select diff_est, dist_good_est,  dist_bad_est, score 
	from output_graph_ds_est  
	where ks_est=(select max(ks_est) from output_graph_ds_est); /* To find the max diff and co-ords*/

	create table maxtab as
	  select * from maxtab_x
	  where score = (select min(score) from maxtab_x);

	update output_graph_ds_est 
        set maxval_est=(select diff_est from  maxtab),
	dgoods_est=(select dist_good_est from maxtab),
	dbads_est=(select dist_bad_est from  maxtab)

	where score=(select score from  maxtab);
	quit;
		
%end;


proc sort data = output_graph_ds_act;
by score;
run;
proc sort data = output_graph_ds_est;
by score;
run;

data &output_ds;
merge output_graph_ds_act( in= a) output_graph_ds_est (in= b);
if a= 1 and b =1 then output &output_ds;
by score;
run;


%mend csbmva_ks;
