/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_pdlgdccf_backtest_fact.sas
Function       : This macro creates PD/LGD/CCF fact a particular time point for backtesting.

Authors        :  CSB Team
Date           :  10FEB2011
SAS            :  9.2
Called-by      :  Stored proc for backtesting
Calls          :  %RCSET macro to capture the return code for error handling.
				  
Input Datasets :  MODEL_MASTER_EXTENSION (APDM library)
                  _<MODEL_RK>_<ABT_DT>_SCR (BACKTEST library)

Output Datasets:  _<MODEL_RK>_<SCORE_TIME_SK>_OOT_PD_FACT (CSBFACT library)
                                   OR
                  _<MODEL_RK>_<SCORE_TIME_SK>_OOT_LGD_FACT (CSBFACT library)
                                   OR
                  _<MODEL_RK>_<SCORE_TIME_SK>_OOT_CCF_FACT (CSBFACT library)

Logic          :  1. Find  level_sk, target period of the model.
                  2. For the ABT_dt, find corresponding time_sk from time_dim. It will the
                     score_time_sk for the model.
                  3. Find target_time_sk by advancing abt date by target period & taking the
                     the corresponding time_sk from time_dim.
                  4. Depending upon the model_type_cd ,create correnponding pd/lgd/ccf fact.

Parameters     :  MODEL_RK   -> Retained Key of the model for which ABT is to be build.
                  ABT_DT     -> Date for which ABT is to build for backtesting.
                  MODEL_TYPE -> Model type code.

**********************************************************************************************/

%macro csbmva_pdlgdccf_backtest_fact(model_rk = ,
									 abt_dt   = ,
                                     model_type = );

*==================================================;
* Defining local macro variables ;
*==================================================;

%local m_abt_dttm  m_abt_ch_dt     m_level_sk
	   m_tgt_prd   m_tgt_freq_cd  m_cut_off  m_tgt_dttm
	   scr_time_sk tgt_time_sk;


*==================================================================;
* finding the model_type_cd for the model ;
*==================================================================;

proc sql noprint;
	select model_type_cd into :model_type
	from &csb_apdm_libref..model_master_extension 
	where model_key = &model_rk;
quit;

%let model_type = &model_type;
%let model_type = %str(%')&model_type%str(%');

*==================================================================;
* Getting level name of the model ;
*==================================================================;

  %local level_var_nm;
  proc sql noprint;
    select kupcase(LEVEL_KEY_COLUMN_NM) into :level_var_nm
    from &dabt_lib..model_master_extension as a, &dabt_lib..level_master as b
    where a.level_sk = b.level_sk
    and   a.MODEL_KEY    = &model_rk;
  quit;
  %bankfdn_rcset(&sqlrc);

*==================================================;
* Finding  level_sk ;
*==================================================;

data _null_;
    dt = &abt_dt;
	ch_dt = put(dt,date9.);
	dttm = dhms(dt,23,59,59);
    
	call symput('m_abt_dttm',dttm);
	call symput('m_abt_ch_dt',ch_dt);
run;
%bankfdn_rcset(&syserr);

proc sql noprint;
  select  level_sk, target_period_cnt, target_period_time_freq_cd,
  		  cut_off_score into
          :m_level_sk, :m_tgt_prd, :m_tgt_freq_cd, :m_cut_off
  from &dabt_lib..model_master_extension
  where model_key = &model_rk
  ;
quit;
%let m_tgt_freq_cd = &m_tgt_freq_cd;

*==================================================;
* Finding score_time_sk & target_time_sk ;
*==================================================;

%let target_prd_adj_cnt = 0;
%if &model_type eq &LGD_MODEL %then %do;
	%let target_prd_adj_cnt = &CSB_DEF_CPTR_PERIOD_POST_LGD_SCR;
%end;

data _null_;
  tgt_dt = intnx("&m_tgt_freq_cd",&abt_dt,&m_tgt_prd + &target_prd_adj_cnt, 'end');
  tgt_dttm = dhms(tgt_dt,23,59,59);
  call symput('m_tgt_dttm',tgt_dttm);
run;

proc sql noprint;
  select time_sk into :scr_time_sk
  from dim.time_dim
  where PERIOD_FIRST_DTTM <= &m_abt_dttm <= PERIOD_LAST_DTTM;
quit;

proc sql noprint;
  select time_sk into :tgt_time_sk
  from dim.time_dim
  where PERIOD_FIRST_DTTM <= &m_tgt_dttm <= PERIOD_LAST_DTTM;
quit;

%let scr_time_sk = &scr_time_sk;
%let tgt_time_sk = &tgt_time_sk;

*==================================================;
* Creating PD fact ;
*==================================================;

%if &model_type = &PD_APP_MODEL or &model_type = &PD_ONG_MODEL
%then %do;

  data csbfact._&model_rk._&scr_time_sk._oot_pd_fact;
    length  &LEVEL_VAR_NM	ESTIMATED_PD 		SCORE_POINTS
			ACTUAL_GOOD_BAD PREDICTED_GOOD_BAD
		    MODEL_RK        LEVEL_SK        
		    SCORE_TIME_SK   TARGET_TIME_SK 8;
    set backtest._&model_rk._&m_abt_ch_dt._scr;

	if score_points >= &m_cut_off > . then
	  predicted_good_bad = 0;
	else
      predicted_good_bad = 1;

	score_time_sk = &scr_time_sk;
	target_time_sk = &tgt_time_sk;
	model_rk = &model_rk;

	level_sk = &m_level_sk;

	keep &LEVEL_VAR_NM  ESTIMATED_PD SCORE_POINTS ACTUAL_GOOD_BAD PREDICTED_GOOD_BAD
		 MODEL_RK   	LEVEL_SK     SCORE_TIME_SK
		 TARGET_TIME_SK;
	rename &LEVEL_VAR_NM = RECORD_RK;
  run;

  proc sql;
    create unique index record_rk on 
	  csbfact._&model_rk._&scr_time_sk._oot_pd_fact(record_rk);
  quit;

%end;

*==================================================;
* Creating LGD fact ;
*==================================================;

%if &model_type = &LGD_MODEL %then %do;

  data backtest_lgd_fact;
    set backtest._&model_rk._&m_abt_ch_dt._scr;

    recovery_end_dt = intnx("&m_tgt_freq_cd",default_dt,&m_tgt_prd);

  run;

  /* Finding Default_time_sk & Recovery_end_time_sk */
  proc sql;
    create table backtest_lgd_fact1 as
      select a.*,
             b.time_sk as default_time_sk,
	  	     c.time_sk as recovery_end_time_sk
	  from backtest_lgd_fact a
	          left join
		   dim.time_dim b
	  on (datepart(b.PERIOD_FIRST_DTTM) <= default_dt <= datepart(b.PERIOD_LAST_DTTM))
			  left join   
		  dim.time_dim c
	  on (datepart(c.PERIOD_FIRST_DTTM) <= recovery_end_dt <= datepart(c.PERIOD_LAST_DTTM))
    ;
  quit;

  data csbfact._&model_rk._&scr_time_sk._oot_lgd_fact;
    length &LEVEL_VAR_NM	 ESTIMATED_LGD  ACTUAL_LGD ACTUAL_LOSS EXPOSURE_AT_DEFAULT
	       DEFAULT_TIME_SK   RECOVERY_END_TIME_SK
		   MODEL_RK          LEVEL_SK        SCORE_TIME_SK
		   TARGET_TIME_SK 8;
    set backtest_lgd_fact1;

	if actual_loss = . then actual_loss = 0;

	score_time_sk = &scr_time_sk;
	target_time_sk = &tgt_time_sk;
	model_rk = &model_rk;
	
	level_sk = &m_level_sk;

	keep &LEVEL_VAR_NM	 ESTIMATED_LGD  ACTUAL_LGD ACTUAL_LOSS EXPOSURE_AT_DEFAULT
	     DEFAULT_TIME_SK RECOVERY_END_TIME_SK
		 MODEL_RK        LEVEL_SK        SCORE_TIME_SK
		 TARGET_TIME_SK;
	rename &LEVEL_VAR_NM = RECORD_RK;
  run;

  proc sql;
    create unique index record_rk on 
	  csbfact._&model_rk._&scr_time_sk._oot_lgd_fact(record_rk);
  quit;

%end;

*==================================================;
* Creating CCF fact ;
*==================================================;

%if &model_type = &CCF_MODEL %then %do;

  data csbfact._&model_rk._&scr_time_sk._oot_ccf_fact;
    length &LEVEL_VAR_NM     ESTIMATED_CCF  ACTUAL_CCF  
		   MODEL_RK          LEVEL_SK    SCORE_TIME_SK
		   TARGET_TIME_SK 8;
    set backtest._&model_rk._&m_abt_ch_dt._scr;

    score_time_sk = &scr_time_sk;
	target_time_sk = &tgt_time_sk;
	model_rk = &model_rk;
	
	level_sk = &m_level_sk;

	keep &LEVEL_VAR_NM  ESTIMATED_CCF  ACTUAL_CCF  
		 MODEL_RK       LEVEL_SK       SCORE_TIME_SK
		 TARGET_TIME_SK;
	rename &LEVEL_VAR_NM = RECORD_RK;
  run;

  proc sql;
    create unique index record_rk on 
	  csbfact._&model_rk._&scr_time_sk._oot_ccf_fact(record_rk);
  quit;

%end;

%mend csbmva_pdlgdccf_backtest_fact;
