/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_modeldetail_scoresummary.sas
Function       : This macro creates ANALYTICAL_MODEL_SUMMARY dataset for the details of
                 model. It also creates the datasets for each combination of <MODEL_TYPE_CD>
                 and <PRODUCT_TYPE_CD> which shows the health of model for each scoring run. 

Authors        :  CSB Team
Date           :  17AUG2010
SAS            :  9.2

Called-by      :  Mid - tier 
Calls          :  %bankfdn_debug_lib_path macro to set debug library.

Input Datasets :  ANALYTICAL_MODEL_DIM (Dim library)
                  LEVEL_MASTER (DABT library)
                  POOL_SCHEME (CSB DIM library)
                  MODEL_SCORE_SUMMARY (CSB FACT library)

Output Datasets:  ANALYTICAL_MODEL_SUMMARY (WORK library)
                  <MODEL_TYPE_CD>_<PRODUCT_TYPE_CD>_SCORE_SUMMARY (WORK library)

Logic          :  1. Extract the valid records as on current datetime from
                     ANALYTICAL_MODEL_DIM table and join with LEVEL_MASTER & POOL_SCHEME
                     to create ANALYTICAL_MODEL_SUMMARY table.
                  2. Get distinct combination of MODEL_TYPE_CD & PRODUCT_TYPE_CD.
                  3. For each combination filter the records from MODEL_SCORE_SUMMARY table
                  4. Transpose the subsetted dataset horizontally on SCORE_TIME to create
                     the output dataset.

Parameters     :  None
**********************************************************************************************/

%macro csbmva_ui_model_score_summary(user = );

options mprint mlogic nolabel;

*==============================================================;
* Setting up the log path to physical folder ;
*==============================================================;

data _NULL_;
	name=put(time(),mmss8.1);
	name1=kscan(name,1,":.");
	name2=kscan(name,2,":.");
	name3=kscan(name,3,":.");
	name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));
	call symput('log_name',"&LOG_PATH_CS"||"&sysmacroname"||ktrim(kleft(name4))||".log");
run;

*==============================================================;
* Creating local macro variables ;
*==============================================================;

%local m_model_type_cd_lst m_model_type_cd_cnt
       m_product_cd_lst    m_product_cd_cnt
       m_model_type_cd     m_product_cd
       m_model_rk_lst
	   i                   j;

*==============================================================;
* Setting up debug library ;
*==============================================================;

%local debug_lib;
%let debug_lib = ;
%bankfdn_debug_lib_path;

*==============================================================;
* Creating ANALYTICAL_MODEL_SUMMARY dataset ;
*==============================================================;

proc sql;
  create table work.Analytical_model_summary_prep as
    select a.MODEL_KEY as MODEL_RK, a.MODEL_NM, a.MODEL_DESC,
           a.MODEL_TYPE_CD, put(a.MODEL_TYPE_CD,$modtype.) as MODEL_TYPE_DESC length = 100,
           a.MODEL_PRODUCT_TYPE_CD,put(a.MODEL_PRODUCT_TYPE_CD,$prtype.) as MODEL_PRODUCT_TYPE_DESC length = 100,
           a.DEPLOYMENT_STATUS_CD, put(a.DEPLOYMENT_STATUS_CD,$moddep.) as DEPLOYMENT_STATUS_DESC length = 100,
           a.LEVEL_SK, b.LEVEL_SHORT_NM,
           a.TARGET_PERIOD_CNT as TARGET_PERIOD_CNT_TMP, a.TARGET_PERIOD_TIME_FREQ_CD, d.TIME_FREQUENCY_SHORT_NM,
           a.CUT_OFF_SCORE, a.OFFSET_RT, a.FACTOR,
           c.POOL_SCHEME_NM, c.POOL_SCHEME_TYPE_CD, 
           c.POOL_SCHEME_TYPE_CD_PARAM, put(c.POOL_SCHEME_TYPE_CD_PARAM,$poolschtype.) as POOL_SCHEME_TYPE_DESC,
           a.CREATED_DTTM
	from &csb_apdm_libref..model_master_extension as a 
              left join &DABT_LIB..level_master as b
    on (a.level_sk = b.level_sk)
              left join &csb_apdm_libref..pool_scheme as c
    on (a.pool_scheme_sk = c.pool_scheme_sk)
	          left join &DABT_LIB..time_frequency_master as d
	on (a.TARGET_PERIOD_TIME_FREQ_CD = d.time_frequency_cd)
	where kupcase(ktrim(kleft(a.created_by))) = "%kupcase(&user)" 
    and a.MODEL_TYPE_CD IN (&PD_ONG_MODEL., &PD_APP_MODEL., &LGD_MODEL.,&CCF_MODEL.);
quit;

*==============================================================;
* Creating a dataset with all model_type ;
*==============================================================;

proc format library = format fmtlib cntlout = model_type_master;
  select $modtype;
run;

data model_type_master;
  set model_type_master(keep = start label rename = (start = MODEL_TYPE_CD
                                                     label = MODEL_TYPE_DESC));

  if kcompress(MODEL_TYPE_CD) in ( %kupcase(&PD_ONG_MODEL.) %kupcase(&PD_APP_MODEL.)
                                   %kupcase(&LGD_MODEL.) %kupcase(&CCF_MODEL.) );

  if kcompress(MODEL_TYPE_CD) = %kupcase(&PD_ONG_MODEL.) then do;
    MODEL_TYPE_CD_PARAM = 'PD_ONG_MODEL';
    display_order = 2;
  end;
  else if kcompress(MODEL_TYPE_CD) = %kupcase(&PD_APP_MODEL.) then do;
    MODEL_TYPE_CD_PARAM = 'PD_APP_MODEL';
    display_order = 1;
  end;
  else if kcompress(MODEL_TYPE_CD) = %kupcase(&LGD_MODEL.) then do;
    MODEL_TYPE_CD_PARAM = 'LGD_MODEL';
    display_order = 3;
  end;
  else if kcompress(MODEL_TYPE_CD) = %kupcase(&CCF_MODEL.) then  do;
    MODEL_TYPE_CD_PARAM = 'CCF_MODEL';
    display_order = 4;
  end;


  attrib _all_ label = '';
run;

*==============================================================;
* Populating MODEL_TYPE_CD_PARAM column ;
*==============================================================;

data Analytical_model_summary_prep;
  set Analytical_model_summary_prep;
  length MODEL_TYPE_CD_PARAM $32;

  if kcompress(MODEL_TYPE_CD) = %kupcase(&PD_ONG_MODEL.) then
    MODEL_TYPE_CD_PARAM = 'PD_ONG_MODEL';
  else if kcompress(MODEL_TYPE_CD) = %kupcase(&PD_APP_MODEL.) then
    MODEL_TYPE_CD_PARAM = 'PD_APP_MODEL';
  else if kcompress(MODEL_TYPE_CD) = %kupcase(&LGD_MODEL.) then
    MODEL_TYPE_CD_PARAM = 'LGD_MODEL';
  else if kcompress(MODEL_TYPE_CD) = %kupcase(&CCF_MODEL.) then
    MODEL_TYPE_CD_PARAM = 'CCF_MODEL';

run;

*==============================================================;
* Populating BACKTEST_IND column ;
*==============================================================;
 
proc sql;
  create table Analytical_model_summary_prep2 as
    select a.*,case when b.model_rk = . then "%sysfunc(sasmsg(smd_ds.bismsg,CRS.BACKTEST_FLAG_N.1, noquote))"
			   else "%sysfunc(sasmsg(smd_ds.bismsg,CRS.BACKTEST_FLAG_Y.1, noquote))" end 
			   as BACKTEST_FLG length = 32
	from Analytical_model_summary_prep as a left join
		 (select distinct model_rk
          from &CS_FACT_LIB..model_backtest_summary) b
    on (a.model_rk = b.model_rk);
quit;

*==============================================================;
* Populating Target period columns for CCF models ;
*==============================================================;

proc sql;
	create table product_recovery as select 
 	dequote(PARAMETER_VALUE) as product_type_cd,
   	ccf_outcome_period ,
   	ccf_outcome_period_days ,
   	ccf_outcome_period_weeks 
   	from &dabt_lib..Product_recovery_config a, custpara.parameter b
   	where kcompress(kupcase(a.product_type_cd_param)) = kcompress(kupcase(b.parameter_nm));
quit;

proc sql;
	insert into product_recovery values
	("&OTHER_PRD_TYPE",&recovery_period_ccf_month,&recovery_period_ccf_day,.);
quit;

proc sql;
	  create table ccf_outcome as
      select model_rk,
            CASE WHEN TARGET_PERIOD_TIME_FREQ_CD = 'MONTH' THEN ccf_outcome_period
			WHEN TARGET_PERIOD_TIME_FREQ_CD = 'DAY' THEN ccf_outcome_period_days
			WHEN TARGET_PERIOD_TIME_FREQ_CD = 'WEEK' THEN ccf_outcome_period_weeks
			ELSE 0 END as Target_period_ccf
	  from Analytical_model_summary_prep2 a,
		   Product_recovery b
	  where kcompress(kupcase(MODEL_PRODUCT_TYPE_CD)) = product_type_cd
	  and kcompress(kupcase(model_type_cd)) = &CCF_MODEL.
		;
quit;

proc sql;
	create table Analytical_model_summary as
    select a.*,
	case when kcompress(kupcase(model_type_cd)) = &CCF_MODEL. then b.Target_period_ccf
	else a.TARGET_PERIOD_CNT_TMP end as TARGET_PERIOD_CNT
	from Analytical_model_summary_prep2 as a left join
		 ccf_outcome b
    on (a.model_rk = b.model_rk);

quit;



*==============================================================;
* Getting the list of MODEL_TYPE_CD & MODEL_PRODUCT_TYPE_CD;
*==============================================================;

proc sql noprint;

  select distinct(MODEL_TYPE_CD),count(distinct MODEL_TYPE_CD)
    into :m_model_type_cd_lst separated by ' ', :m_model_type_cd_cnt
  from work.Analytical_model_summary;

  select distinct(MODEL_PRODUCT_TYPE_CD), count(distinct MODEL_PRODUCT_TYPE_CD) 
    into :m_product_cd_lst separated by ' ', :m_product_cd_cnt
  from work.Analytical_model_summary;

quit;

*==============================================================;
* Getting model key of MODEL_MASTER_EXTENSION in ;
* model_score_summary dataset ;
*==============================================================;

proc sql;
  create table &debug_lib..model_score_summary as
    select c.model_key as model_rk, a.score_time_sk,
		   a.clr_model
	from &CS_FACT_LIB..model_score_summary a,
		 (select distinct model_rk, source_model_master_key
		  from dim.analytical_model_dim) b,
		 &csb_apdm_libref..model_master_extension c
	where a.model_rk = b.model_rk
	and   b.source_model_master_key = c.model_key
	order by c.model_key, score_time_sk;
quit;

*==============================================================;
* Creating o/p dataset for each MODEL_TYPE_CD;
* & MODEL_PRODUCT_TYPE_CD;
*==============================================================;

%do i = 1 %to &m_model_type_cd_cnt;
 
  %let m_model_type_cd = %kscan(&m_model_type_cd_lst,&i,%str( ));

  %do j = 1 %to &m_product_cd_cnt;

    %let m_product_cd = %kscan(&m_product_cd_lst,&j,%str( ));
    %let m_model_rk_lst = .;

	proc sql noprint;
	  select model_rk into :m_model_rk_lst separated by ' '
	  from work.Analytical_model_summary
	  where MODEL_TYPE_CD = "&m_model_type_cd" and
            MODEL_PRODUCT_TYPE_CD = "&m_product_cd";
	quit;

	data &debug_lib..model_score_summary_subset;
	  set &debug_lib..model_score_summary;
 
      if model_rk in (&m_model_rk_lst);

	run;

	proc sql;
	  create table &debug_lib..model_score_summary_subset1 as
	    select c.MODEL_RK, c.MODEL_NM, c.TARGET_PERIOD_TIME_FREQ_CD, c.DEPLOYMENT_STATUS_CD,
		       a.clr_model , datepart(b.period_last_dttm) as score_time_point  
        from &debug_lib..model_score_summary_subset as a left join dim.time_dim as b
        on (a.score_time_sk = b.time_sk) inner join work.Analytical_model_summary as c
        on  (a.model_rk = c.model_rk)
      order by MODEL_RK, score_time_point;
	quit;

	%let out_table_name = &m_model_type_cd._&m_product_cd._score_summary;

	proc transpose data = &debug_lib..model_score_summary_subset1 out = &out_table_name(drop = _name_) prefix = _;
	  by MODEL_RK MODEL_NM TARGET_PERIOD_TIME_FREQ_CD DEPLOYMENT_STATUS_CD;
      id score_time_point;
      var clr_model;
    run;

  %end;

%end;

*==============================================================;
* Resetting log path to default ;
*==============================================================;
options label;

%mend csbmva_ui_model_score_summary;
